import 'package:meta/meta.dart';

import '../turtle_commands.dart';
import '../turtle_state.dart';
import '_exceptions.dart';
import '_instructions.dart';

/// Runs a macro by a given [name].
@immutable
class RunMacro implements TurtleCommand {
  /// Name of the macro.
  final String name;

  /// The arguments.
  final Function(Map) macroArgv;

  /// Creates a new instance.
  RunMacro(this.name, this.macroArgv);

  @override
  List<Instruction> createInstruction(TurtleState turtle, Map argv) {
    if (name == null) {
      throw Exception('Macro does not exist');
    }

    var macro = turtle.macros[name];
    if (macro == null) {
      throw Exception('Macro does not exist');
    }

    final width = turtle.strokeWidth;
    final color = turtle.color;
    final degrees = turtle.degrees;
    final position = turtle.position;

    var instructions = <Instruction>[];
    var copy = Map.of(argv);
    if (this.macroArgv != null) {
      copy.addAll(this.macroArgv(argv));
    }
    try {
      var list = List<Instruction>.of(macro.commands
          .map((command) => command.createInstruction(turtle, copy))
          .expand((x) => x));
      instructions.addAll(list);
    } on StopException catch (_) {}

    instructions.addAll(<Instruction>[
      SetStrokeWidthInstruction(width),
      SetColorInstruction(color),
    ]);

    turtle.degrees = degrees;
    turtle.position = position;
    return instructions;
  }
}
